package com.example.rabbitmq;

import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.DeliverCallback;

public class Worker {

    private final static String QUEUE_NAME = "MBServer";


    /*
     méthode principal
     @param argv -parametre en ligne de commande.
    */
    public static void main(String[] argv) throws Exception {
        /*
        *Crée la connection avec le serveur.
        */
        ConnectionFactory factory = new ConnectionFactory();
        factory.setHost("localhost");
        Connection connection = factory.newConnection();
        Channel channel = connection.createChannel();

        channel.queueDeclare(QUEUE_NAME, false, false, false, null);
        System.out.println(" [*] Waiting for messages. To exit press CTRL+C");

        channel.basicQos(1);

        DeliverCallback deliverCallback = (consumerTag, delivery) -> {
            String message = new String(delivery.getBody(), "UTF-8");

            System.out.println(" [x] Received '" + message + "'");
            try {
                //Split message " "
                //Switch case sur message
                //
                String mots[] = message.split(" ");
                switch(mots[0]){

                    /**
                    *Switch méthode afin de savoir la tache à effectuée
                    */
                    //Création d'un étudiant ou d'un club
                    case "CREATE":
                        System.out.println("Bonjour");
                        System.out.println(mots[1]);
                        if(mots[1].equals("CLUB")){
                            String club = "";
                            for (int i=2;i<mots.length;i++){
                                club += mots[i];
                            }
                            String methode = "POST";
                            String uri = "http://localhost:8080/clubs";
                            String jsonInputString = "{\"name\":\"" + club +"\"}";
                            sendRequest(methode,uri,jsonInputString);
                            System.out.println("Message : " + jsonInputString + " envoyé a " + uri);
                        }
                        else if(mots[1].equals("STUDENT")){
                            String student = "";
                            for (int i=2;i<mots.length;i++){
                                student += mots[i];
                            }
                            String methode = "POST";
                            String uri = "http://localhost:8080/student";
                            String jsonInputString = "{\"name\" : \"" + student +"\"}";
                            sendRequest(methode,uri,jsonInputString);
           
                        }else{
                            System.out.println("Choix incorrect, veuillez utilisez '-h' pour voir les options disponibles ");
                        }
                        break;

                    //Ajout d'un étudiant à un club
                    case "ADD":
                        System.out.println("Hello");
                        if(mots[1].equals("STUDENT")){
                            String student = "";
                            int i=2;
                            while(!mots[i].equals("TOCLUB")){
                                student += mots[i];   //8
                                i++;
                            }
                            String club = "";
                            for(int j=i+1; j< mots.length; j++){
                                club += mots[j];
                            }
                            String methode = "POST";
                            String uri = "http://localhost:8080/clubs";
                            String jsonInputString = "{\"name\" : \"" + club +"\", \"preside\":{ \"name\" : \"" + student +"\"} }";
                            sendRequest(methode,uri,jsonInputString);
                        }
                        break;

                    //Indication d'execution
                    case "-h":
                        System.out.println(" Invalid commande send ");
                        System.out.println("  Please use : ");
                        System.out.println("CREATE CLUB tata");
                        System.out.println("CREATE STUDENT toto");
                        System.out.println("ADD STUDENT toto TOCLUB tata");
                        break;

                    default:
                        System.out.println("Choix incorrect, veuillez utilisez '-h' pour voir les options disponibles ");
                        
                        break;
                }
                doWork(message);
            } catch (Exception e) {
                throw new RuntimeException(e);
            } finally {
                System.out.println(" [x] Done");
                channel.basicAck(delivery.getEnvelope().getDeliveryTag(), false);
            }
        };
        boolean autoAck = true; // acknowledgment is covered below
        channel.basicConsume(QUEUE_NAME, autoAck, deliverCallback, consumerTag -> { });
    }

    /*
    * Méthode d'execution de la tache.
    * 
    * @param task - the task to be run
    */
    private static void doWork(String task) {
        for (char ch : task.toCharArray()) {
            if (ch == '.') {
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException _ignored) {
                    Thread.currentThread().interrupt();
                }
            }
        }
    }

    /*
    * Envoi une JSON request à l'adresse URI donnée.
    * 
    * @param methode - method to send the request to the server
    * @param uri - the URI of the request
    * @param json - JSON string to send to the server
    */
    private static void sendRequest(String methode, String uri, String json) throws Exception {
        URL url;
        try {
            url = new URL(uri);
            HttpURLConnection http = (HttpURLConnection)url.openConnection();
            http.setRequestMethod(methode);
            http.setDoOutput(true);
            http.setRequestProperty("Content-Type", "application/json");
            http.setRequestProperty("Content-Length", "0");
            String jsonInputString = json;
            try(OutputStream os = http.getOutputStream()){
                byte[] input = jsonInputString.getBytes("utf-8");
                os.write(input, 0, input.length);
                System.out.println("Message envoyé....");
            }
            System.out.println(http.getResponseCode() + " " + http.getResponseMessage());
            http.disconnect();
        } catch (Exception e) {
            throw new Exception(e);
        }
    }
}
