# REST Service + test

by Mickael Riviere & Baptiste Lebreton

## Dépendences pour ce projet
JDK 17 mais doit fonctionner avec un JDK plus ancien >11

* Spring Web
* Spring JPA Data
* Spring Boot DevTools
* MariaDB Driver
* TestContainers + MariaDB
* Lombok

Nécessite une installation de Docker pour les tests avec MariaDB

## Consigne
Pour ce TP vous allez devoir assimiler beaucoup de notions. Vous pouvez vous appuyer sur ce qui a été expliqué en cours
et sur les exemples donnés dans le code. Vous trouverez des ressources également dans le fichier HELP.md 
Essayez de comprendre ce qui se passe, l'assemblage des composants, les dépendances et ne faites pas simplement du copier/coller



## Architecture



![Schema](./Schema.png)

## A faire pour ce TP
* Installation de l'image docker de StudentRest
* Tester le client HTTP pour faire des requêtes directement sur les API (pensez à la BD)
  * Vous devez d'abord démarrer l'application (run ResttestApplication)
  * utilisez curl ou le HTTPClient de IntelliJ (Tools>HTTP Client)
* Complétez le StudentController et les méthodes correspondantes, en particulier celles pour gérer la relation entre Student et Club et écrivez des tests.
  * Complétez le StudentController
  * AJoutez ce qui est nécessaire pour permettre de gérer en respectant les principes de REST la relation entre Student et Club
  * Complétez les tests de StudentService
  * Ajoutez des classe de test pour StudentController
* Création d'une classe Message Client permettant émettre des messages avec des commandes de créations de club, d’étudiants et de relations entre eux.
* Création d'une classe Worker permettant de réceptionner les messages et d'envoyer les commandes à l'API
* Mise en place de RabbitMQ



## Lancement de notre système

- Premièrement, exécution de docker comprenant le RabbitMQ grâce à la commande :

​		`docker compose up tpdirest-master`

- On doit ensuite compiler les deux classe ClientMessage et Worker grâce à la commande suivante
  `javac ClientMessage.java Worker.java`
- On peut maintenant lancer nos différent programme :
  - On commence par notre API :
    `java ResttestApplication` `
  - On exécute ensuite nos deux worker :
    ```java worker```
    ```java worker```
  - Enfin, on peut envoyer nos messages grace à ClientMessage:
    - Plusieurs opération sont possible :
      - CREATE STUDENT _nom_student_         \\\ *Afin d'ajouter un étudiant dans la base*
      - CREATE CLUB _nom_club_                       \\\ *Afin d'ajouter un club dans la base*
      - ADD STUDENT _nom_student_ TOCLUB _nom_club_   \\\ *Afin d'ajouter un étudient qui préside un club*
    - Exemple de commande de création d'étudiant :
      `java ClientMessage CREATE STUDENT toto`
    - Exemple de commande de création de club :
      `java ClientMessage CREATE CLUB iLoveJava`
    - Exemple de commande de création d'étudiant qui préside un cub :
      `java ClientMessage ADD STUDENT toto TOCLUB iLoveJava`

## Amélioration du système

*Que peut-on faire pour améliorer l’architecture et dispatcher les opérations en fonction de leur nature (READ/WRITE) ?*



*Quelle autre solution permettrait d’améliorer les performances de ce système pour réduire la charge du serveur et de la base de données. Faites le schéma d’architecture correspondant.*

On peut mettre en place du Load-balancing, cela permet de réduire la charge sur le serveur et les effets bottle-neck.



