package eu.telecomnancy.resttest;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;


@RestController
public class StudentController {
    @Autowired
    private StudentService studentService;
    @Autowired
    private ClubService clubService;

    /* Avec GET et ce mapping on récupère tous les clubs exisants. */
    @GetMapping("student")
    public Iterable<StudentModel> getAllStudents(){
        return studentService.getAllStudents().stream().map(StudentModel::new).collect(Collectors.toSet());
    }

    /* Avec POST et ce mapping on créé un nouveau club passé dans le corps de la requête. */
    @PostMapping("/student")
    public ResponseEntity<StudentModel> createStudent(@RequestBody StudentModel newStudent) {
        System.out.println("createStudent "+newStudent);
        return ResponseEntity.ok(new StudentModel(studentService.createStudent(newStudent.getName())));
    }


    /* Avec Get et ce mapping on ne récupère que le club dont l'id ou le nom est précisé. */
    @GetMapping("/student/{id}")
    public ResponseEntity<StudentModel> getStudent(@PathVariable Long id) {
        return ResponseEntity.ok(new StudentModel(studentService.findById(id)));
    }

    /* Avec PUT et ce mapping on peut mettre a jour le club dont l'id est précisé ou le créer si il n'existe pas
     à partir des éléments dans le corps de la requête. */
    @PutMapping("/student/{id}")
    public ResponseEntity<StudentModel> replaceStudent(@RequestBody StudentModel newStudent, @PathVariable Long id) {
         return ResponseEntity.ok(new StudentModel(studentService.updateStudent(id, newStudent.getName())));
    }
    



    /* Avec DELETE et ce mapping on supprime le club dont l'id est précisé. */
    @DeleteMapping("/student/{id}")
    public void deleteClub(@PathVariable Long id) {
        studentService.deleteStudent(id);
    }
}
