package eu.telecomnancy.resttest;

import static org.assertj.core.api.Assertions.assertThat;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

@SpringBootTest
public class StudentControllerMockTests {

    @Autowired
    private StudentController studentController;
    
    @MockBean
    private StudentRepository studentRepository;

    @MockBean
    private ClubRepository clubRepository;
    
    @BeforeEach
    public void setUp() {
        /* Création de deux étudiants pour la plupart des tests */
        Student alex = new Student("Alexandre");
        alex.setId(1L);
        Optional<Student> alexo=Optional.of(alex);
        Student maxime = new Student("Maxime");
        maxime.setId(2L);
        Optional<Student> maximo=Optional.of(maxime);

        /* Création d'une list pour le test de all() */
        List<Student> list = new ArrayList<>();
        list.add(alex);
        list.add(maxime);

        /* Simulation des méthodes des repository via Mockito */

        /* Création d'un club pour addClub() */
        Club club = new Club("Test");
        club.setId(1L);

        /* Mocking du Student alex */
        Mockito.when(studentRepository.findByName(alex.getName())).thenReturn(alexo);
        Mockito.when(studentRepository.findById(alex.getId())).thenReturn(alexo);

        /* Mocking du Student maxime */
        Mockito.when(studentRepository.findByName(maxime.getName())).thenReturn(maximo);
        Mockito.when(studentRepository.findById(maxime.getId())).thenReturn(maximo);

        /* Mocking de la méthode findAll() */
        Mockito.when(studentRepository.findAll()).thenReturn(list);

        /* Mocking de la méthode count() */
        Mockito.when(studentRepository.count()).thenReturn((long) list.size());

        /* Mocking nécessaire pour modify() et addClub()  */
        Mockito.when(studentRepository.save(alex)).thenReturn(alex);
        Mockito.when(clubRepository.findById(club.getId())).thenReturn(Optional.of(club));
        Mockito.when(clubRepository.save(club)).thenReturn(club);
        Mockito.when(clubRepository.findByName(club.getName())).thenReturn(Optional.of(club));
    }

    @Test
    public void whenGetName_ReturnAllStudents() {
        /* Test de la méthode getAllStudents() */

        List<StudentModel> list = new ArrayList<>();
        Student alexS = new Student("Alexandre");
        alexS.setId(1L);
        StudentModel alex = new StudentModel(alexS);
        Student maxS = new Student("Maxime");
        maxS.setId(2L);
        StudentModel max = new StudentModel(maxS);
        list.add(alex);
        list.add(max);
        
        List<StudentModel> foundList = new ArrayList<StudentModel>();
        
        Iterable<StudentModel> found = studentController.getAllStudents();
        Iterator<StudentModel> s = found.iterator();
        
        while(s.hasNext()) {
        	foundList.add(s.next());
        }
        
        assertThat(foundList).isEqualTo(list);
    }
/*
    @Test
    public void createStudent_sucessfully() {
    	Student Ant = new Student("Zargluck");
    	StudentModel st = new StudentModel(Ant);
    	ResponseEntity<StudentModel> newStudent_isok = studentController.createStudent(st);
    	assertThat(newStudent_isok).isNotNull();
		assertThat(newStudent_isok.getStatusCode()).isEqualTo(HttpStatus.OK);
		assertThat(newStudent_isok.getBody().getName()).isEqualTo("Zargluck");
    	
    }*/


    @Test
    public void getStudent_sucessfully() {
    	ResponseEntity<StudentModel> newStudent_isok = studentController.getStudent(1L);
    	assertThat(newStudent_isok).isNotNull();
		assertThat(newStudent_isok.getStatusCode()).isEqualTo(HttpStatus.OK);
		assertThat(newStudent_isok.getBody().getName()).isEqualTo("Alexandre");
    	
    }

    @Test
    public void replaceStudent_sucessfully() {
        Student alex = new Student("Alexandre");
        alex.setId(1L);
    	StudentModel stModel1 = new StudentModel(alex);
        Student maxime = new Student("Maxime");
        maxime.setId(1L);
    	StudentModel stModel2 = new StudentModel(maxime);
    	//studentController.createStudent(stModel1);
    	
    	ResponseEntity<StudentModel> newStudent_isok = studentController.replaceStudent(stModel2, 1L);
    	
    	assertThat(newStudent_isok).isNotNull();
		assertThat(newStudent_isok.getStatusCode()).isEqualTo(HttpStatus.OK);
		assertThat(newStudent_isok.getBody().getName()).isEqualTo("Ulrik");
    	
    }
}
