package eu.telecomnancy.resttest;

import static org.assertj.core.api.Assertions.assertThat;

import java.util.Iterator;

import javax.persistence.EntityManager;
import javax.sql.DataSource;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.jdbc.core.JdbcTemplate;

@SpringBootTest
public class StudentControllerTests {

    @Autowired
    private DataSource dataSource;
    @Autowired private JdbcTemplate jdbcTemplate;
    @Autowired private EntityManager entityManager;
    @Autowired private StudentRepository studentRepository;
    @Autowired private StudentService studentService;
    @Autowired private ClubRepository clubRepository;
    @Autowired private RoomRepository roomRepository;

    @Autowired
    private StudentController studentController;
    
    @Test
    void contextLoads() {
    }

    @Test
    public void injectedComponentsAreNotNull(){
        assertThat(dataSource).isNotNull();
        assertThat(jdbcTemplate).isNotNull();
        assertThat(entityManager).isNotNull();
        assertThat(studentRepository).isNotNull();
    }
    
    @Test
    public void getAllStudents_returnStudents() {
        studentRepository.save(new Student("Antoine"));
        studentRepository.save(new Student("Bernard"));
    	Iterable<StudentModel> students = studentController.getAllStudents();
    	Iterator<StudentModel> s = students.iterator();
    	assertThat(s.hasNext());
    	
    }

    @Test
    public void createStudent_sucessfully() {
    	StudentModel st = new StudentModel(new Student("Cédric"));
    	ResponseEntity<StudentModel> newStudent_isok = studentController.createStudent(st);
    	assertThat(newStudent_isok).isNotNull();
		assertThat(newStudent_isok.getStatusCode()).isEqualTo(HttpStatus.OK);
		assertThat(newStudent_isok.getBody().getName()).isEqualTo("Cédric");
    	
    }

    @Test
    public void getStudent_sucessfully() {
    	Student st = new Student("Dimitri");
        studentRepository.save(st);
    	ResponseEntity<StudentModel> newStudent_isok = studentController.getStudent(st.getId());
    	assertThat(newStudent_isok).isNotNull();
		assertThat(newStudent_isok.getStatusCode()).isEqualTo(HttpStatus.OK);
		assertThat(newStudent_isok.getBody().getName()).isEqualTo("Dimitri");
    	
    }
    

    @Test
    public void replaceStudent_sucessfully() {
    	Student st1 = new Student("Edouardo");
        studentRepository.save(st1);
    	StudentModel stModel1 = new StudentModel(st1);
    	StudentModel stModel2 = new StudentModel(new Student("Fabrice"));
    	//studentController.createStudent(stModel1);
    	
    	ResponseEntity<StudentModel> newStudent_isok = studentController.replaceStudent(stModel2, st1.getId());
    	
    	assertThat(newStudent_isok).isNotNull();
		assertThat(newStudent_isok.getStatusCode()).isEqualTo(HttpStatus.OK);
		assertThat(newStudent_isok.getBody().getName()).isEqualTo("Fabrice");
    	
    }
}
